#include "hope.h"

void init(void)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_FLAT);
  /* When initialized, have location on origin,
     looking down positive Z-axis where up is positive-Y axis. */
  p1.position[0] = p1.position[1] = p1.position[2]= 0;
  p1.direction[0] = p1.direction[1] = 0;
  p1.roll[0] = p1.roll[2] = 0;
  p1.roll[1] = p1.direction[2] = 1;
}

void camera(void)
{
  gluLookAt(p1.position[0], p1.position[1], p1.position[2],
	    p1.direction[0], p1.direction[1], p1.direction[2],
	    p1.roll[0], p1.roll[1], p1.roll[2]);
}

void drawGeometry(void)
{
  int x,y,z;
  for (x = 0; x < WORLD_WIDTH; x++)
  {
    for (y = 0; y < WORLD_WIDTH; y++)
    {
      for (z = 0; z < WORLD_WIDTH; z++)
      {
	if (v[x][y][z])
	{
	  glPushMatrix();
	  glTranslatef(x - 10, y - 10, z - 10);
	  glutSolidCube(1.0);
	  glPopMatrix();
	}
      }
    }
  }  
}

// copied from swiftless tutorial
void enable(void)
{
  glEnable (GL_DEPTH_TEST); //enable the depth testing
  glEnable (GL_LIGHTING); //enable the lighting
  glEnable (GL_LIGHT0); //enable LIGHT0, our Diffuse Light
  glShadeModel (GL_SMOOTH); //set the shader to smooth shader
}

void display(void)
{
  //glClear(GL_COLOR_BUFFER_BIT);
  glColor3f(1.0, 1.0, 1.0);
  glClearColor(0,0,0,1);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //clear the color buffer and the depth buffer
  glLoadIdentity();
  camera();
  //enable();
  //gluLookAt (eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
  //glScalef(1.0, 1.0, 1.0);
  drawGeometry();
  glFlush();
}

void reshape (int w, int h)
{
  glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  float ytoxr = 1.5; // default value of 1.5 should never be used
  if (w != 0 && h != 0)
    ytoxr = (float)h / (float)w;
  glFrustum(-1.0, 1.0, -ytoxr, ytoxr, 1.5, 100);
  //glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
  glMatrixMode (GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
  double angle = M_PI / 32;
  
  switch (key)
  {
    case 'p':
      printf("position[x] = %f\nposition[y] = %f\nposition[z] = %f\n",
	     p1.position[0], p1.position[1], p1.position[2]);
      printf("direction[x] = %f\ndirection[y] = %f\ndirection[z] = %f\n",
	     p1.direction[0], p1.direction[1], p1.direction[2]);
      printf("roll[x] = %f\nroll[y] = %f\nroll[z] = %f\n",
	     p1.roll[0], p1.roll[1], p1.roll[2]);
      break;
    case 'w': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "forward"); break;
    case 's': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "backward"); break;
    case 'a': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "left"); break;
    case 'd': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "right"); break;
    case ' ': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "up"); break;
    case 'c': vectorTranslate(p1.position, p1.direction, p1.roll, 1, "down"); break;
    case 'r': vectorRotate(p1.direction, p1.roll, "roll", angle); break;
    case 't': vectorRotate(p1.direction, p1.roll, "roll", -angle); break;
    case 'g': vectorRotate(p1.direction, p1.roll, "pitch", angle); break;
    case 'h': vectorRotate(p1.direction, p1.roll, "pitch", -angle); break;
    case 'e': vectorRotate(p1.direction, p1.roll, "yaw", angle); break;
    case 'v': vectorRotate(p1.direction, p1.roll, "yaw", - angle); break;
    case 'q': exit(EXIT_SUCCESS); break;
  }
}

int main (int argc, char * argv[])
{
  int x,y,z;
  
  bool i[WORLD_WIDTH][WORLD_WIDTH];
  fractInit(WORLD_WIDTH, i);
  
  for (y=0;y<WORLD_WIDTH;y++)
  {
    for (x=0;x<WORLD_WIDTH;x++)
    {
      for (z=0;z<WORLD_WIDTH;z++)
      {
	v[x][y][z] = (i[x][y] && i[z][y]);
      }
    }
  }
  
  glutInit(&argc, argv);
  glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
  glutInitWindowSize (500, 500); 
  glutInitWindowPosition (100, 100);
  glutCreateWindow (argv[0]);
  init ();
  glutDisplayFunc(display);
  glutIdleFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard);
  glutMainLoop();
  
  return(EXIT_SUCCESS);
}

void fractInit(int size, bool i[][size])
{
  int x, y;
  for (x = 0; x < size; x++)
  {
    for (y = 0; y < size; y++)
    {
      i[x][y] = ((x % 3) == 0) && ((y % 4) == 0);
    }
  }
}

/*
  Convention for writing cubes is:
    Bottom of cube: 0 1 2 3 where 0 is (0,0,0)
    Top of cube: 7 6 5 4 where 5 is (1,1,1)
  The order of operations is significant when Blender
  tries to read the .obj output file. Stay safe.
 */
void writeCube(FILE * fout, int x, int y, int z, int r)
{
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + r);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + r);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 2, totalVerts + 1, totalVerts + 0);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 7, totalVerts + 6, totalVerts + 5, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 0, totalVerts + 1, totalVerts + 6, totalVerts + 7);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 0, totalVerts + 7, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 2, totalVerts + 3, totalVerts + 4, totalVerts + 5);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 1, totalVerts + 2, totalVerts + 5, totalVerts + 6);
  totalVerts++;
}

void writeVertex(FILE * fout, int x, int y, int z)
{
  fprintf(fout, "v %d %d %d\n", x, y, z);
}
