#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>

/** PHYSICS. All physics-related math functions should
   stay within this area to stay organized.
   All functions return 0 on normal exit, 1 on error.
   I have chosen to avoid returning vectors and quaternions
   to force the programmer to calloc/free their data in the
   same block of code.
   Good references:
    http://mathworld.wolfram.com/Quaternion.html
    http://local.wasp.uwa.edu.au/~pbourke/geometry/rotate/
    http://en.wikipedia.org/wiki/Cross_product
    http://mathworld.wolfram.com/CrossProduct.html
    http://msdn.microsoft.com/en-us/library/cc500340.aspx
    http://www.j3d.org/matrix_faq/matrfaq_latest.html
    http://www.gamasutra.com/view/feature/3278/rotating_objects_using_quaternions.php
    http://gpwiki.org/index.php/OpenGL:Tutorials:Using_Quaternions_to_represent_rotation
    */

/* Rotate a vector in any of the following directions:
   "pitch", "roll", or "yaw".
   You may abbreviate this as "r", "p", and "y".
   Note that these rotations build on the current rotation. */
int vectorRotate(GLdouble * forward3d, GLdouble * up3d, char * direction, const double angle);

/* Translate a vector in any of the following directions.
   "up", "down", "left", "right", "forward", "backward" (or "u", "d", "l", "r", "f", "b").
   IMPORTANT: these movements are relative to your current rotation, so
   if the up3d vector has you turned upside down and you want to tell this to go up,
   then your actual (global) movement will be downwards. This is the behavior
   expected by the player. */
int vectorTranslate(GLdouble * location3d, GLdouble * forward3d, GLdouble * up3d, const GLdouble radius, char * direction);
